@@ -15,5 +15,5 @@ extern int check_emulation(int her_major_version,
 			    int *return_major,
 			    int *return_minor);
 
-#endif //EMULATE_H
+#endif /* EMULATE_H */
 
