@@ -1,4 +1,4 @@
-/* maximum file descriptors = 3240 */
+/* maximum file descriptors = 680 */
 
 struct Min_Auth_Fd {
   int fd;
@@ -8,6 +8,8 @@ struct Min_Auth_Fd {
 static struct Min_Auth_Fd mafd[] = {
   { 4, "sh" },
   { 64, "csh" },
+  { 64, "tcsh" },
+  { 4, "bash" },
   { 64, "zsh" },
   { 0, "" },
 };
