@@ -13,6 +13,25 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 
 /*
  * $Log: userfile.c,v $
+ * Revision 1.9  1996/10/04 01:01:49  kivinen
+ * 	Added printing of path to fatal calls in userfile_open and and
+ * 	userfile_local_socket_connect. Fixed userfile_open to
+ * 	userfile_local_socket_connect in calls to fatal in
+ * 	userfile_local_socket_connect.
+ *
+ * Revision 1.8  1996/09/27 17:18:06  ylo
+ * 	Fixed a typo.
+ *
+ * Revision 1.7  1996/09/08 17:21:08  ttsalo
+ * 	A lot of changes in agent-socket handling
+ *
+ * Revision 1.6  1996/09/04 12:39:59  ttsalo
+ * 	Added connecting to unix-domain socket
+ *
+ * Revision 1.5  1996/08/13 09:04:19  ttsalo
+ * 	Home directory, .ssh and .ssh/authorized_keys are now
+ * 	checked for wrong owner and group & world writeability.
+ *
  * Revision 1.4  1996/05/29 07:37:31  ylo
  * 	Do setgid and initgroups when initializing to read as user.
  *
@@ -105,7 +124,22 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 
        20 USERFILE_PCLOSE_REPLY
 	  int32		return value
-*/
+
+       21 USERFILE_LOCAL_SOCK
+          string        path
+
+       22 USERFILE_LOCAL_SOCK_REPLY
+          int32         handle (-1 if error)
+	  
+       23 USERFILE_SEND
+	  int32		handle
+	  int32         flags
+	  string	data
+
+       24 USERFILE_SEND_REPLY
+	  int32		bytes_written  ;; != length of data means error
+	
+	  */
 
 #include "includes.h"
 #include <gmp.h>
@@ -137,6 +171,10 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 #define USERFILE_POPEN_REPLY   18
 #define USERFILE_PCLOSE	       19
 #define USERFILE_PCLOSE_REPLY  20
+#define USERFILE_LOCAL_SOCK    21 
+#define USERFILE_LOCAL_SOCK_REPLY 22
+#define USERFILE_SEND          23
+#define USERFILE_SEND_REPLY    24
 
 
 /* Flag indicating whether we have forked. */
@@ -350,6 +388,7 @@ static void userfile_child_server()
   char *path, *cp, *command;
   char buf[8192];
   struct stat st;
+  struct sockaddr_un sunaddr;
 
   for (;;)
     {
@@ -399,6 +438,20 @@ static void userfile_child_server()
 	  xfree(cp);
 	  break;
 
+	case USERFILE_SEND:
+	  handle = buffer_get_int(&packet);
+	  flags = buffer_get_int(&packet);
+	  cp = buffer_get_string(&packet, &len);
+
+	  ret = send(handle, cp, len, flags);
+
+	  userfile_packet_start(USERFILE_SEND_REPLY);
+	  buffer_put_int(&packet, ret);
+	  userfile_packet_send();
+
+	  xfree(cp);
+	  break;
+
 	case USERFILE_CLOSE:
 	  handle = buffer_get_int(&packet);
 
@@ -489,6 +542,30 @@ static void userfile_child_server()
 
 	  break;
 
+	case USERFILE_LOCAL_SOCK:
+	  path = buffer_get_string(&packet, NULL);
+
+	  sunaddr.sun_family = AF_UNIX;
+	  strncpy(sunaddr.sun_path, path, sizeof(sunaddr.sun_path));
+
+	  ret = socket(AF_UNIX, SOCK_STREAM, 0);
+	  if (ret >= 0)
+	    {
+	      if (connect(ret, (struct sockaddr *)&sunaddr,
+			  AF_UNIX_SIZE(sunaddr)) < 0)
+		{
+		  close(ret);
+		  ret = -1;
+		}
+	    }
+
+	  userfile_packet_start(USERFILE_LOCAL_SOCK_REPLY);
+	  buffer_put_int(&packet, ret);
+	  userfile_packet_send();
+
+	  xfree(path);
+	  break;
+
 	default:
 	  fatal("userfile_child_server: packet type %d", type);
 	}
@@ -573,7 +650,7 @@ void userfile_init(const char *username, uid_t uid, gid_t gid,
 
 void userfile_close_pipes()
 {
-  if (!userfile_open)
+  if (!userfile_initialized)
     return;
   userfile_initialized = 0;
   close(userfile_fromchild);
@@ -620,6 +697,12 @@ static UserFile userfile_make_handle(int type, int handle)
   return uf;
 }
 
+/* Encapsulate a normal file descriptor inside a struct UserFile */
+UserFile userfile_encapsulate_fd(int fd)
+{
+  return userfile_make_handle(USERFILE_LOCAL, fd);
+}
+
 /* Opens a file using the given uid.  The uid must be either the current
    effective uid (in which case userfile_init need not have been called) or
    the uid passed to a previous call to userfile_init.  Returns a pointer
@@ -639,12 +722,12 @@ UserFile userfile_open(uid_t uid, const char *path, int flags, mode_t mode)
     }
 
   if (!userfile_initialized)
-    fatal("userfile_open: using non-current uid but not initialized (uid=%d)",
-	  (int)uid);
+    fatal("userfile_open: using non-current uid but not initialized (uid=%d, path=%.50s)",
+	  (int)uid, path);
   
   if (uid != userfile_uid)
-    fatal("userfile_open: uid not current and not that of child: uid=%d",
-	  (int)uid);
+    fatal("userfile_open: uid not current and not that of child: uid=%d, path=%.50s",
+	  (int)uid, path);
 
   userfile_packet_start(USERFILE_OPEN);
   buffer_put_string(&packet, path, strlen(path));
@@ -660,6 +743,50 @@ UserFile userfile_open(uid_t uid, const char *path, int flags, mode_t mode)
   return userfile_make_handle(USERFILE_REMOTE, handle);
 }
 
+UserFile userfile_local_socket_connect(uid_t uid, const char *path)
+{
+  int handle;
+  struct sockaddr_un sunaddr;
+  
+  if (uid == geteuid())
+    {
+      sunaddr.sun_family = AF_UNIX;
+      strncpy(sunaddr.sun_path, path, sizeof(sunaddr.sun_path));
+
+      handle = socket(AF_UNIX, SOCK_STREAM, 0);
+      if (handle < 0)
+	return NULL;
+
+      if (connect(handle, (struct sockaddr *)&sunaddr,
+		  AF_UNIX_SIZE(sunaddr)) < 0)
+	{
+	  close(handle);
+	  return NULL;
+	}
+            
+      return userfile_make_handle(USERFILE_LOCAL, handle);
+    }
+
+  if (!userfile_initialized)
+    fatal("userfile_local_socket_connect: using non-current uid but not initialized (uid=%d, path=%.50s)",
+	  (int)uid, path);
+  
+  if (uid != userfile_uid)
+    fatal("userfile_local_socket_connect: uid not current and not that of child: uid=%d, path=%.50s",
+	  (int)uid, path);
+
+  userfile_packet_start(USERFILE_LOCAL_SOCK);
+  buffer_put_string(&packet, path, strlen(path));
+  userfile_packet_send();
+
+  userfile_packet_read(USERFILE_LOCAL_SOCK_REPLY);
+  handle = buffer_get_int(&packet);
+  if (handle < 0)
+    return NULL;
+
+  return userfile_make_handle(USERFILE_REMOTE, handle);
+}
+
 /* Closes the userfile handle.  Returns >= 0 on success, and < 0 on error. */
 
 int userfile_close(UserFile uf)
@@ -821,6 +948,52 @@ int userfile_write(UserFile uf, const void *buf, unsigned int len)
     }
 }
 
+/* Writes data to the file.  Writes all data, unless an error is encountered.
+   Returns the number of bytes actually written; -1 indicates error. */
+
+int userfile_send(UserFile uf, const void *buf, unsigned int len,
+		  unsigned int flags)
+{
+  unsigned int chunk_len, offset;
+  int ret;
+  const unsigned char *ucp;
+
+  switch (uf->type)
+    {
+    case USERFILE_LOCAL:
+      return send(uf->handle, buf, len, flags);
+      
+    case USERFILE_REMOTE:
+      ucp = buf;
+      for (offset = 0; offset < len; )
+	{
+	  chunk_len = len - offset;
+	  if (chunk_len > 16000)
+	    chunk_len = 16000;
+	  
+	  userfile_packet_start(USERFILE_SEND);
+	  buffer_put_int(&packet, uf->handle);
+	  buffer_put_int(&packet, flags);
+	  buffer_put_string(&packet, ucp + offset, chunk_len);
+	  userfile_packet_send();
+	  
+	  userfile_packet_read(USERFILE_SEND_REPLY);
+	  ret = buffer_get_int(&packet);
+	  if (ret < 0)
+	    return -1;
+	  offset += ret;
+	  if (ret != chunk_len)
+	    break;
+	}
+      return offset;
+
+    default:
+      fatal("userfile_send: type %d", uf->type);
+      /*NOTREACHED*/
+      return 0;
+    }
+}
+
 /* Reads a line from the file.  The line will be null-terminated, and
    will include the newline.  Returns a pointer to the given buffer,
    or NULL if no more data was available.  If a line is too long,
@@ -1019,3 +1192,16 @@ int userfile_pclose(UserFile uf)
       return -1;
     }
 }
+
+int userfile_check_owner_permissions(struct passwd *pw, const char *path)
+{
+  struct stat st;
+  if (userfile_stat(pw->pw_uid, path, &st) < 0)
+    return 0;
+
+  if ((st.st_uid != 0 && st.st_uid != pw->pw_uid) ||
+      (st.st_mode & 022) != 0)
+    return 0;
+  else
+    return 1;
+}
